# -*- coding: utf-8 -*-
"""
Created on Sun Jan 12 22:50:12 2025

@author: mstep
"""

import math

def get_star_points(start_theta, num_points, skip, bbox):
    '''Generate the points for a star centered in the bounding box.'''
    # bbox = (xmin, ymin, xmax, ymax)
    wid = bbox[2] - bbox[0]
    hgt = bbox[3] - bbox[1]
    rx = wid / 2
    ry = hgt / 2
    cx = bbox[0] + rx
    cy = bbox[1] + ry

    points = []

    # Make sure skip is between 0 and num_points - 1.
    skip = skip % num_points

    # Is skip > num_points / 2, use the smaller equivalent.
    if skip > num_points / 2:
        skip = num_points - skip

    # Do not allow skip = 0.
    if skip == 0:
        raise ValueError('skip should not be 0 or a multiple of num_points')

    # If this is a polygon, don't bother with concave points.
    if skip == 1:
        theta = start_theta
        dtheta = 2 * math.pi / num_points
        for i in range(num_points):
            point = (cx + rx * math.cos(theta),
                     cy + ry * math.sin(theta))
            points.append(point)
            theta += dtheta
        return points

    # Find the radius for the concave vertices.
    inner_fraction = calculate_inner_radius(num_points, skip)

    # Make the points.
    theta = start_theta
    dtheta = math.pi / num_points
    for i in range(num_points):
        points.append((
            cx + rx * math.cos(theta),
            cy + ry * math.sin(theta)))
        theta += dtheta
        points.append((
            cx + rx * math.cos(theta) * inner_fraction,
            cy + ry * math.sin(theta) * inner_fraction))
        theta += dtheta
    return points

def calculate_inner_radius(num_points, skip):
    '''Calculate the inner star radius.'''
    # If skip = num_points / 2, make a fake star instead of an asterisk.
    if math.isclose(skip, num_points / 2):
        return 0.33

    # Calculate angles to key points.
    dtheta = 2 * math.pi / num_points
    theta00 = -math.pi / 2
    theta01 = theta00 + dtheta * skip
    theta10 = theta00 + dtheta
    theta11 = theta10 - dtheta * skip

    # Find the key points.
    pt00 = (math.cos(theta00), math.sin(theta00))
    pt01 = (math.cos(theta01), math.sin(theta01))
    pt10 = (math.cos(theta10), math.sin(theta10))
    pt11 = (math.cos(theta11), math.sin(theta11))

    # See where the segments connecting the points intersect.
    lines_intersect, segments_intersect, intersection, close_p1, close_p2 = \
        find_line_intersection(pt00, pt01, pt10, pt11)

    # Calculate the distance between the point of intersection and the center.
    return math.sqrt(
        intersection[0] * intersection[0] +
        intersection[1] * intersection[1])

def find_line_intersection(p1, p2, p3, p4):
    '''
    Find the point of intersection between the lines p1 --> p2 and p3 --> p4.
    Return:
        lines_intersect - True if the lines intersect
        segments_intersect - True if the segments intersect
        intersection - The point of lines' intersection
        close_p1 - The point on line 1 closest to the intersection
        close_p2 - The point on line 2 closest to the intersection
    '''
    # Get the segments' parameters.
    dx12 = p2[0] - p1[0]
    dy12 = p2[1] - p1[1]
    dx34 = p4[0] - p3[0]
    dy34 = p4[1] - p3[1]

    # Solve for t1 and t2
    denominator = dy12 * dx34 - dx12 * dy34

    try:
        t1 = ((p1[0] - p3[0]) * dy34 + (p3[1] - p1[1]) * dx34) / denominator
    except ZeroDivisionError:
        # The lines are parallel (or close enough to it).
        lines_intersect = False
        segments_intersect = False
        intersection = None
        close_p1 = None
        close_p2 = None
        return lines_intersect, segments_intersect, intersection, \
            close_p1, close_p2
    except Exception as ex:
        print(f'Unexpected error, {type(ex).__name__}: {ex}')
        raise

    lines_intersect = True

    t2 = ((p3[0] - p1[0]) * dy12 + (p1[1] - p3[1]) * dx12) / -denominator

    # Find the point of intersection.
    intersection = (p1[0] + dx12 * t1, p1[1] + dy12 * t1)

    # The segments intersect if t1 and t2 are between 0 and 1.
    segments_intersect = ((t1 >= 0) and (t1 <= 1) and (t2 >= 0) and (t2 <= 1))

    # Find the closest points on the segments.
    if t1 < 0:
        t1 = 0
    elif t1 > 1:
        t1 = 1

    if t2 < 0:
        t2 = 0
    elif t2 > 1:
        t2 = 1

    close_p1 = (p1[0] + dx12 * t1, p1[1] + dy12 * t1)
    close_p2 = (p3[0] + dx34 * t2, p3[1] + dy34 * t2)

    return lines_intersect, segments_intersect, intersection, close_p1, close_p2

#%%
import tkinter as tk
from PIL import Image, ImageDraw, ImageTk

class FillStarApp:
    def __init__(self):
        self.window = tk.Tk()

        self.window.title('pil_fill_star')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('300x300')

        # Load the fill image.
        fill_image = Image.open('robot.jpg')
        wid = fill_image.width
        hgt = fill_image.height

        # Create the result image with a light yellow background.
        result_image = Image.new('RGB', (wid, hgt), 'lightyellow')

        # Create a Draw object to draw on the result image.
        result_dr = ImageDraw.Draw(result_image)

        # Draw something to be the background.
        radius = math.sqrt(3) * max(wid, hgt) - 10
        skip = 3
        color = 'red'
        for degrees in range(0, 90, skip):
            radians = math.radians(degrees)
            x = radius * math.cos(radians)
            y = radius * math.sin(radians)
            result_dr.line([(0, 0), (x, y)], fill=color)
            dx = radius * math.cos(math.pi / 2 + radians)
            dy = radius * math.sin(math.pi / 2 + radians)
            result_dr.line([(wid, 0), (wid + dx, dy)], fill=color)

        # Get the star's points.
        bbox = (0, 0, wid - 1, hgt - 1)
        points = get_star_points(-math.pi / 2, 7, 2, bbox)

        # Create a mask from the star.
        # L = 8-bit grayscale. 1 = one bit per pixel.
        mask_image = Image.new('1', (wid, hgt), 'black')
        #mask_image = Image.new('L', (wid, hgt), 'black')
        #mask_image = Image.new('L', (wid, hgt), 100)

        # Make a Draw object to draw on the mask.
        mask_dr = ImageDraw.Draw(mask_image)

        # Fill the mask with white.
        mask_dr.polygon(points, fill='white')

        # Save the mask image so we can see what it looks like.
        mask_image.save('mask.png')

        # Paste the fill image onto the base image using the mask
        # None means upper left corner is (0, 0).
        result_image.paste(fill_image, (None), mask_image)

        # Draw the an outline around the star.
        result_dr.polygon(points, outline='blue', width=5)

        # Save the result_image (just to show we can).
        result_image.save('result.jpg')

        # Convert the image into a PhotoImage for tkinter to display.
        photo_image = ImageTk.PhotoImage(result_image)

        # Create the canvas sized to fit the image.
        self.canvas = tk.Canvas(self.window, bg='white', width=wid, height=hgt,
            borderwidth=0, highlightthickness=0)
        margin = 10
        self.canvas.pack(side=tk.TOP, anchor=tk.NW, padx=margin, pady=margin)

        # Display the image on the canvas.
        self.canvas.create_image(0, 0, image=photo_image, anchor=tk.NW) 

        # Size the window to fit.
        self.window.geometry(f'{wid + 2 * margin}x{hgt + 2 * margin}')

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

#%%
FillStarApp()
print('Done')
